/**********************************************************************************************
Copyright (c) 2013 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_create_pool_feed_ds.sas
Function       : This macro creates the feed table for PD\LGD\CCF Model on pool level.

Authors        :  CSB Team
Date           :  28JUN2013
SAS            :  9.4

Called-by      :  ETL jobs POOL_CREATE_FEED_PD/POOL_CREATE_FEED_LGD/POOL_CREATE_FEED_CCF. 
Calls          :  %BANKFDN_RCSET macro to capture the return code for error handling.

Input Datasets :  _<pool_schme_sk>_<score_time_sk>_pool_fact (CSBFACT library)
                  analytical_model_dim (DIM library)
		  time_dim (DIM library)
  		 		  pool_dim (APDM library)
		          model_x_pool(APDM library)
				  model_master_extension (APDM library)
                  _<model_rk>_<score_time_sk>_<PD\LGD\CCF>_FACT (CSBFACT library)
                  

Output Datasets:  _<MODEL_RK>_<SCORE_TIME_SK>_<PD\LGD\CCF>_POOL_FD (CSB FACT library)

Logic          :  1.Get mm_ext_key from dim.analytical_model_dim.
                  2. Get scoring date time using score_time_sk from Time_dim.
	          3. Get level info using model_master_extension.
                  4. Using pool_fact tables, allocate pools for this pool_scheme using pool_dim.
                  5. Using _pool_fact and _PD\lgd\ccf_fact , derive actual information.
                  6. Based on model_type create output datasets.

Parameters     :  input_ds_lib     -> input dataset library for fact tables.
                  output_ds_lib    -> output library where feed table will be created.
                  output_ds_sufx   -> output dataset suffix used for e.g. _pool_fd.
                  pool_scheme_sk   -> pool scheme sk for which this feed needs to be generated.
                  score_time_sk    -> time key for whcih this feed needs to be generated.
                  model_rk         -> model identifier refers to model_rk of analytical_model_dim.
		          model_type_cd    -> type of the model i.e. PDO,PDA,LGD,CCF.
						

**********************************************************************************************/


%macro csbmva_create_pool_feed_ds(input_ds_lib=,output_ds_lib=,output_ds_sufx=,pool_scheme_sk=,score_time_sk=,model_rk=,model_type_cd= );

*==================================================================;
* Initialize parameters ;
*==================================================================;

%let output_ds_sufx=%lowcase(&output_ds_sufx);
%let inp_tblname=_&pool_scheme_sk._&score_time_sk._pool_fact;
%let model_type_cd=%sysfunc(dequote(&model_type_cd));


*==================================================================;
* Creating separate dataset for ACCEPTED application ;
*==================================================================;

%local model_type;
proc sql;
	select model_type_cd into :model_type
	from &csb_apdm_libref..POOL_SCHEME a, &csb_apdm_libref..MODEL_MASTER_EXTENSION b
	where a.pd_model_rk  = b.model_key
	and		pool_scheme_sk = &pool_scheme_sk.;
quit;

%let model_type = &model_type;
%let model_type = %str(%')&model_type%str(%');

%if &model_type eq &PD_APP_MODEL %then %do;
	proc sql;
		create table inp_table_accpt_population as
			select
				a.*
			from 
				&input_ds_lib..&inp_tblname a,
				dim.application_dim b
			where a.record_rk = b.application_rk
			and kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT
			and valid_end_dttm = &valid_high_dttm;
	quit;

	%let inp_table_accpt_ds_lib = work;
	%let inp_table_accpt_ds_nm = inp_table_accpt_population;

%end;
%else %do;

	%let inp_table_accpt_ds_lib = &input_ds_lib.;
	%let inp_table_accpt_ds_nm = &inp_tblname;

%end;

*==================================================================;
* get scoring date time corresponding to score_time_sk ;
*==================================================================;
  Proc sql;
    	select  PERIOD_LAST_DTTM  format=best. into :scr_dt	 
  		from
      		dim.TIME_DIM 
                where TIME_SK=&score_time_sk
   ;
quit;


%bankfdn_rcset(&sqlrc);
%let scr_dt=&scr_dt;

*==================================================================;
* Find  mm_ext_key and level_sk for pool scheme ;
*==================================================================;
%local mm_ext_key;
PROC SQL;
	select source_model_master_key into: mm_ext_key
	from dim.analytical_model_dim
    where model_rk=&model_rk
        and &scr_dt between valid_start_dttm and valid_end_dttm
    ;
QUIT;	
%bankfdn_rcset(&sqlrc);

%let mm_ext_key=&mm_ext_key;
%let scr_dt=&scr_dt;
PROC SQL;
	select level_sk into :level_sk
	from &csb_apdm_libref..model_master_extension
    where model_key=&mm_ext_key
 ;
QUIT;
*==================================================================;
* Group _<pool_scheme>_<score_time>_pool_fact on pool_sk to calculate stats ;
*==================================================================;

proc sql;
	create table pool_dim_ext as
		select 
			pool_sk, pool_name, 
			no_of_records as no_of_records_dev			
		from &csb_apdm_libref..pool_dim
		where pool_scheme_sk = &pool_scheme_sk.
		and &scr_dt between valid_start_dttm and valid_end_dttm;
quit;

%if &model_type= &PD_APP_MODEL  %then %do;
	proc sql;
		create table ds_for_records_actual_all as
			select
				pool_sk,
				count(*) as no_of_records_actual_all
			from &input_ds_lib..&inp_tblname
			group by pool_sk;
	quit;
%end;

proc sql;
	create table ds_for_records_actual_accpt as
		select
			pool_sk,
			count(*) as no_of_records_actual
		from &inp_table_accpt_ds_lib..&inp_table_accpt_ds_nm
		group by pool_sk;
quit;

PROC SQL;
	create table group_fact as
		select
			a.pool_sk,
			a.pool_name,
			a.no_of_records_dev,
			%if &model_type= &PD_APP_MODEL  %then %do;
				
				coalesce(no_of_records_actual_all,0) as no_of_records_actual_all,
			%end;
			coalesce(no_of_records_actual,0) as no_of_records_actual
	from pool_dim_ext a
		%if &model_type= &PD_APP_MODEL  %then %do;
			 	left join ds_for_records_actual_all b
    		on 	 ( a.pool_sk = b.pool_sk )
		%end;
				left join ds_for_records_actual_accpt c
			on 	( a.pool_sk = c.pool_sk ) ;
QUIT;
%bankfdn_rcset(&sqlrc);	

%let mtype=%str(%')&model_type_cd%str(%') ;

*==================================================================;
* Mapping time_sk to datetime value ;
*==================================================================;

data _null_;
  a = put(&score_time_sk,Cs_period_last_dttm_fmt.);
  call symput('period_end_dttm',a);
run;
%bankfdn_rcset(&syserr);


*==================================================================;
* Read the no of variables associated with the pool_scheme ;
*==================================================================;
Proc sql noprint;
	select INPUTVARCOUNT into :varcount
	from &csb_apdm_libref..Pool_scheme_variable_map
	where pool_scheme_sk=%eval(&pool_scheme_sk);
quit;
%bankfdn_rcset(&sqlrc);
%let varcount=&varcount;
*=====================================================================;
* Get the joining dataset name and keys for _model_scoretime_pd_fact ;
*=====================================================================;
%let flag=;
%if &varcount > 1 %then %do ;/* if no of variables on which pooling is based is multiple*/

* get associated levels with the models on which pooling is done ;

 Proc sql noprint;
    select count(distinct mm_ext_key) into :mod_count from csbridge.Analytical_model_x_pool_scheme
        where pool_scheme_sk=%eval(&pool_scheme_sk)
        and &period_end_dttm between valid_start_dttm and valid_end_dttm;
  
	%let mod_count=&mod_count;

    select  distinct mm_ext_key,model_type_cd into :m_rk1 - :m_rk&mod_count,:mtyp1 -:mtyp&mod_count
	from csbridge.Analytical_model_x_pool_scheme a,dim.analytical_model_dim b
	where a.pool_scheme_sk=%eval(&pool_scheme_sk)
	and &period_end_dttm between a.valid_start_dttm and a.valid_end_dttm
	and a.mm_ext_key=b.source_model_master_key
        and &period_end_dttm between b.valid_start_dttm and b.valid_end_dttm
        order by b.model_type_cd descending;	
 quit;
%bankfdn_rcset(&sqlrc);

%do md_cnt=1 %to &mod_count;
   Proc sql noprint;
   	select distinct a.level_sk,kupcase(b.level_key_column_nm) into :level_key_&md_cnt,:level_var_&md_cnt
	from &csb_apdm_libref..model_master_extension a,&csb_apdm_libref..level_master as b
	where a.level_sk=b.level_sk
        and  model_key=&&m_rk&md_cnt;
   Quit;
 %let level_key_&md_cnt=&&level_key_&md_cnt;
 %let level_var_&md_cnt=&&level_var_&md_cnt;
%end;

/* Check which levels are unequal, accordingly set a processing flag*/
   %if (&mod_count eq 2) %then %do;
		%if (&level_key_1)  eq (&level_key_2) %then %let flag=alleq;
   		%else %let flag=1not2;
   %end;
   %else %if (&mod_count eq 3) %then %do;
          %if (&level_key_1  eq &level_key_3) and (&level_key_2  eq &level_key_3) %then %let flag=alleq;
          %if (&level_key_2  ne &level_key_3)  %then %let flag=&flag 2not3;
   		  %if (&level_key_1  ne &level_key_3)  %then %let flag=&flag 1not3;
   %end;
  	%if &flag eq alleq %then %do;/*if models on which pooling is done are on same level*/
		%let joining_ds_nm1=;
		%let joining_ds_lib1=;
	%end;/*if models on whcih pooling is done are on same level*/
	%else %do p=1 %to %eval(%sysfunc(countw(&flag)));/*if models on which pooling is done are on different level*/
	   %let pflag=%scan(&flag,&p);
		proc sql noprint;
		     	 select b.source_table_nm, c.library_reference into
			            :joining_ds_nm&pflag,:joining_ds_lib&pflag
			 	 from &csb_apdm_libref..level_assoc a,&csb_apdm_libref..source_table_master b,
		          	  &csb_apdm_libref..library_master c
		     	 where a.mapping_source_table_sk = b.source_table_sk
		     	 and   b.library_sk = c.library_sk
			 	 %if &pflag eq 1not2 %then %do;
		     	 	and a.level_sk = &level_key_2
		      		and a.assoc_level_sk = &level_key_1
			 	 %end;
			 	 %else %if &pflag eq 2not3 %then %do;
		     	 	and a.level_sk = &level_key_3
		      		and a.assoc_level_sk = &level_key_2
			  	%end;
			  	%else %if &pflag eq 1not3 %then %do;
		      		and a.level_sk = &level_key_3
		      		and a.assoc_level_sk = &level_key_1
			  	%end;
				;
			quit;

			%bankfdn_rcset(&sqlrc);
			
			
			/*flag error if joining dataset not found*/
			%if &&joining_ds_nm&pflag eq %then %do;
			    %let msg1= ERROR: %sysfunc(sasmsg(smd_ds.bismsg,CSMART.POOL_ACROSS_LEVEL_ERROR.1, noquote));
				%put &msg1;
				%abort abend;
			%end;

			%let joining_ds_nm&pflag = &&joining_ds_nm&pflag ;
			%let joining_ds_lib&pflag = &&joining_ds_lib&pflag ;

			%if %upcase(&&joining_ds_nm&pflag)=ACCOUNT_DIM and (&level_key_1=2 or &level_key_1=5) %then 
				%let level_var_1=PRIMARY_CUSTOMER_RK;/**When model build on customer level then joining key to be taken**/

	%end;/*if models on which pooling is done are on different level*/
%end;/* if no of variables on which pooling is based is multiple*/

*==================================================================;
* Create feed tables on the basis of model_type ;
*==================================================================;

%if &mtype eq  &PD_APP_MODEL or  &mtype eq  &PD_ONG_MODEL %then %do;/* For model_type -'PDO' and 'PDA' :start*/
   	PROC SQL;
		create table actual_info as 
		select b.pool_sk,max(score_points) as score_points,sum(actual_good_bad) as no_of_actual_bads
                from
        %if &flag eq or &flag eq alleq %then %do;
		 &input_ds_lib.._&model_rk._&score_time_sk._PD_FACT a,
		 &inp_table_accpt_ds_lib..&inp_table_accpt_ds_nm as b
		where a.record_rk=b.record_rk
        %end;
        %else %do;
             %if &mod_count eq 2 %then %do;
                 &input_ds_lib..&inp_tblname b
                 inner join &&joining_ds_lib&flag...&&joining_ds_nm&flag c
		 on (b.record_rk=c.&level_var_2 and (c.valid_start_dttm <= &period_end_dttm <= c.valid_end_dttm))
	         inner join &input_ds_lib.._&model_rk._&score_time_sk._PD_FACT  a
		 on (c.&level_var_1=a.record_rk)
             %end;
             %else %if &mod_count eq 3 %then %do;
                  %if %kindex(&flag,1not3) %then %do;
                   &inp_table_accpt_ds_lib..&inp_table_accpt_ds_nm b
                   inner join &joining_ds_lib1not3..&joining_ds_nm1not3 d
		   on (b.record_rk=d.&level_var_3 and (d.valid_start_dttm <= &period_end_dttm <= d.valid_end_dttm))
		   inner join &input_ds_lib.._&model_rk._&score_time_sk._PD_FACT  a
		   on (d.&level_var_1=a.record_rk)

                  %end;
                  %else %do;
		     &input_ds_lib.._&model_rk._&score_time_sk._PD_FACT a,
		     &inp_table_accpt_ds_lib..&inp_table_accpt_ds_nm as b
		    where a.record_rk=b.record_rk                    
                  %end;        
             %end;	     
        %end;
        group by b.pool_sk;
		      
    QUIT;
%bankfdn_rcset(&sqlrc);

	PROC SQL;
		create table &output_ds_lib.._&model_rk._&score_time_sk._pd&output_ds_sufx. as
			select
				A.POOL_SK as POOL_SK,
				monotonic() as POOL_SEQ_NO,
				A.POOL_NAME as POOL_NAME,
				A.NO_OF_RECORDS_DEV,
				%if &model_type= &PD_APP_MODEL  %then %do;
					
					A.NO_OF_RECORDS_ACTUAL_ALL,
				%end;
				A.NO_OF_RECORDS_ACTUAL,
				coalesce(c.NO_OF_ACTUAL_BADS,0) as NO_OF_ACTUAL_BADS,
				b.rate as ESTIMATED_PD,
				c.score_points as MAX_SCORE_POINTS,
				&score_time_sk as SCORE_TIME_SK,
				&model_rk as MODEL_RK,
				&level_sk  as LEVEL_SK
		from group_fact as a 
			 left join &csb_apdm_libref..model_x_pool b
			 on a.pool_sk=b.pool_sk
			 	and  &scr_dt between b.valid_start_dttm and b.valid_end_dttm
	    		and b.model_rk=&mm_ext_key
			 left join actual_info c
	    	 on a.pool_sk=c.pool_sk		
        ;
		create unique index XPIPD&output_ds_sufx. on &output_ds_lib.._&model_rk._&score_time_sk._pd&output_ds_sufx.(pool_seq_no,score_time_sk,model_rk);
	QUIT;	
%bankfdn_rcset(&sqlrc);

%end;/* For model_type -'PDO' and 'PDA' :end*/


%else %if &mtype eq  &LGD_MODEL  %then %do;/* For model_type -'LGD' :start*/
   	PROC SQL;
		create table actual_info as 
		select b.pool_sk,avg(actual_lgd) as ACTUAL_LGD from
         %if &mod_count eq 3 and %kindex(&flag,2not3) gt 0 %then %do;
                 &input_ds_lib..&inp_tblname b
		inner join &joining_ds_lib2not3..&joining_ds_nm2not3 c
			on (b.record_rk=c.&level_var_3 and (c.valid_start_dttm <= &period_end_dttm <= c.valid_end_dttm))
		inner join &input_ds_lib.._&model_rk._&score_time_sk._LGD_FACT a
			on (c.&level_var_2=a.record_rk)
        %end;
	%else %do;
		 &input_ds_lib.._&model_rk._&score_time_sk._LGD_FACT a,
		     &input_ds_lib..&inp_tblname as b
		where a.record_rk=b.record_rk
        %end;
        group by b.pool_sk;
		      
    QUIT;
%bankfdn_rcset(&sqlrc);
	PROC SQL;
		create table &output_ds_lib.._&model_rk._&score_time_sk._lgd&output_ds_sufx. as
		select A.POOL_SK as POOL_SK,monotonic() as POOL_SEQ_NO,A.POOL_NAME as POOL_NAME,A.NO_OF_RECORDS_DEV,
		A.NO_OF_RECORDS_ACTUAL,coalesce(sum(case when c.actual_lgd > b.rate then 1 else 0 end),0) as NO_OF_ACTUAL_BADS,
		b.rate as ESTIMATED_LGD,C.ACTUAL_LGD,&score_time_sk as SCORE_TIME_SK,
		&model_rk as MODEL_RK,&level_sk  as LEVEL_SK
		from group_fact as a 
			 left join &csb_apdm_libref..model_x_pool b
			 on a.pool_sk=b.pool_sk
	         and  &scr_dt between b.valid_start_dttm and b.valid_end_dttm
			  and b.model_rk=&mm_ext_key
			 left join actual_info c	    
		     on a.pool_sk=c.pool_sk
		
	   
        ;
		create unique index XPILGD&output_ds_sufx. on &output_ds_lib.._&model_rk._&score_time_sk._lgd&output_ds_sufx.(pool_seq_no,score_time_sk,model_rk);
	QUIT;	
%bankfdn_rcset(&sqlrc);

%end;/* For model_type -'LGD' :end*/


%else %if &mtype eq  &CCF_MODEL  %then %do;/* For model_type -'CCF' :start*/
   	PROC SQL;
		create table actual_info as 
		select b.pool_sk,avg(actual_ccf) as ACTUAL_CCF
 		from &input_ds_lib.._&model_rk._&score_time_sk._CCF_FACT a,
		     &input_ds_lib..&inp_tblname as b
		where a.record_rk=b.record_rk
        group by b.pool_sk;
		      
    QUIT;
%bankfdn_rcset(&sqlrc);
	PROC SQL;
		create table &output_ds_lib.._&model_rk._&score_time_sk._ccf&output_ds_sufx. as
		select A.POOL_SK as POOL_SK,monotonic() as POOL_SEQ_NO,A.POOL_NAME as POOL_NAME,A.NO_OF_RECORDS_DEV,
		A.NO_OF_RECORDS_ACTUAL,coalesce(sum(case when c.actual_ccf > b.rate then 1 else 0 end),0) as NO_OF_ACTUAL_BADS,
		b.rate as ESTIMATED_CCF,c.ACTUAL_CCF,&score_time_sk as SCORE_TIME_SK,
		&model_rk as MODEL_RK,&level_sk  as LEVEL_SK
		from group_fact as a  inner join 
                     &csb_apdm_libref..model_x_pool b
			 on a.pool_sk=b.pool_sk 	
				and  &scr_dt between b.valid_start_dttm and b.valid_end_dttm
	    		and b.model_rk=&mm_ext_key
			 left join actual_info c
			 on  a.pool_sk=c.pool_sk	    
        ;
		create unique index XPICCF&output_ds_sufx. on &output_ds_lib.._&model_rk._&score_time_sk._ccf&output_ds_sufx.(pool_seq_no,score_time_sk,model_rk);
	QUIT;	
%bankfdn_rcset(&sqlrc);

%end;/* For model_type -'CCF' :end*/

%mend csbmva_create_pool_feed_ds;
