/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_feed_pd.sas
Function       : This macro creates the feed table for PD Model on pool level.

Authors        :  CSB Team
Date           :  17AUG2010
SAS            :  9.2

Called-by      :  %Csbmva_generate_feed Macro. 
Calls          :  %BANKFDN_RCSET macro to capture the return code for error handling.

Input Datasets :  _<MODEL_RK>_<SCORE_TIME_SK>_PD_FACT (CSB FACT library)
                  _<MODEL_RK>_DEV_FACT (CSB FACT library)
                  MODEL_X_POOL (CSB BRIDGE library)
                  APPLICATION_DIM (DIM library)
                  POOL_DIM (CSB DIM library)

Output Datasets:  _<MODEL_RK>_<SCORE_TIME_SK>_PD_SCR_FD (CSB FACT library)

Logic          :  1. Get the RATE from MODEL_X_POOL table and join with PD fact.
                  2. Group the above table based on pools and derive the variable related to
                     actual sample.
                  3. Get the dev count from MODEL_DEV_FACT.
                  4. Join with the POOL_DIM to get the POOL_SEQ_NO and POOL_NAME.

Parameters     :  MODEL_TIME_KEY     -> Model_RK & Score_Time_SK concatenated by underscore(_).
                  MODEL_TYPE         -> Type code of model e.g. PDA, PDO, LGD, CCF.
                  INPUT_DS_LIB       -> Library where PD fact is present.
                  INPUT_DS_NM        -> Name of PD Fact.
                  OUT_FEED_LIB       -> Library where feed table will be created.
                  FEED_DS_NM_POOL    -> Name of the feed table.
		  ISDEVELOPMENTCYCLE ->	Whether model is in development phase or scoring
						(possible values : true, false)

**********************************************************************************************/

%macro csbmva_feed_pd(model_key    =  ,time_key        =  ,
                      input_ds_lib =  ,input_ds_nm     =  ,
                      out_feed_lib =  ,feed_ds_nm_pool =  ,
                      model_type   =  ,isDevelopmentCycle = );

%let model_key = &model_key;
%let model_type = &model_type;


*==================================================================;
* Creating local macro variables ;
*==================================================================;

%local    m_level_sk mm_ext_key;

*==================================================================;
* Finding the model_key from model_master_extension ;
*==================================================================;

%if &isDevelopmentCycle = true %then %do;
	%let mm_ext_key = &model_key;
%end;
%else %do;
	proc sql noprint;
		select source_model_master_key into :mm_ext_key
		from dim.analytical_model_dim
		where model_rk = &model_key;
	quit;
	%let mm_ext_key = &mm_ext_key;
%end;

*==================================================================;
* finding the level_sk from model_master_extension ;
*==================================================================;

proc sql noprint;
  select level_sk into  :m_level_sk
  from &dabt_lib..model_master_extension 
  where model_key = &mm_ext_key;
quit;

%bankfdn_rcset(&sqlrc);


*==================================================================;
* Finding version no for which score bands must be assigned ;
*==================================================================;

%let max_grp_version_no=;

%if &isDevelopmentCycle = true %then %do;
	proc sql;
   		select max(range_group_version_no) into :max_grp_version_no from &csb_apdm_libref..model_range_detail
   		where model_key=&mm_ext_key ;	
	quit;

	%bankfdn_rcset(&sqlrc);

	%let max_grp_version_no=&max_grp_version_no;
%end;

%else %do;
    %let dev_version_no=;
	Proc sql;
  		select  dev_measure_version_no  into : dev_version_no
  		from csbfact.model_score_summary
  		where model_rk=&model_key
                and SCORE_TIME_SK=&time_key;
	quit;

	%bankfdn_rcset(&sqlrc);

	%let dev_version_no=&dev_version_no;

         %if &dev_version_no ne %then %do;

		Proc sql;
 			select  range_group_version_no into :max_grp_version_no
  			from  &csb_apdm_libref..model_dev_measure_x_spec_map
			where model_rk=&mm_ext_key
     			and dev_measure_version_no=&dev_version_no;
		quit;
 

		%bankfdn_rcset(&sqlrc);
        	%let max_grp_version_no=&max_grp_version_no;
	%end;
      
       %else %do;
                	proc sql;
   				select max(range_group_version_no) into :max_grp_version_no from &csb_apdm_libref..model_range_detail
   				where model_key=&mm_ext_key ;	
			quit;

			%bankfdn_rcset(&sqlrc);

			%let max_grp_version_no=&max_grp_version_no;

       %end; 

%end;
*==================================================;
* Extracting range_detail table for correct version ;
*==================================================;

Data &debug_lib.._&model_key._&time_key._rng_dtl_ext;
	set &csb_apdm_libref..model_range_detail(where=(model_key=&mm_ext_key 
	     and  range_group_version_no=&max_grp_version_no ));
Run;
%bankfdn_rcset(&syserr);

*==================================================;
* Calling interfacing macro ;
*==================================================;

/*Get path for debug library*/
%local debug_path;
%let debug_path=%sysfunc(pathname(&debug_lib));

%let pd_succ_flg=;

%csbmva_dabt_intrfc_apply_bins(m_in_bin_info_tbl_libref=&debug_lib,
							   m_in_bin_info_tbl_nm=_&model_key._&time_key._rng_dtl_ext,  
							   m_in_bin_unique_id_clmn_nm=range_sk,
							   m_in_dtl_tbl_libref=&input_ds_lib,
							   m_in_dtl_tbl_nm=&input_ds_nm,
						       m_out_dtl_tbl_libref=&debug_lib,
							   m_out_dtl_tbl_nm=_&model_key._&time_key._feed_table_act,
							   m_scr_log_file_path=&debug_path,
							   m_scr_lib_ref=&debug_lib,
							   m_model_sk=&mm_ext_key,
							   m_return_success_flg=pd_succ_flg);
%bankfdn_rcset(&syserr);


*=============================================================;
* If interfacing  macro not completed successfully flag error ;
*=============================================================;

%IF &pd_succ_flg ne 0 %then %do;
	%let msg1= ERROR: %sysfunc(sasmsg(smd_ds.bismsg,CSMART.APPLY_BIN_ERROR.1, noquote));
		  		%let msg2= &model_key;
				%put &msg1 &msg2;  
            %ABORT ABEND;
%end;
%bankfdn_rcset(&syserr);

*=============================================================;
* If all accounts were not allocated in specified bins ;
*=============================================================;

%let accounts_with_no_bin=0;
Proc Sql;
	select count(*) into :accounts_with_no_bin from &debug_lib.._&model_key._&time_key._feed_table_act
	where range_sk is null;
Quit;
%let accounts_with_no_bin=&accounts_with_no_bin;

%bankfdn_rcset(&sqlrc);

*=============================================================;
* Load data into exception tabel and abort the process ;
*=============================================================;

%IF &accounts_with_no_bin ne 0 %then %do;
	%let msg1= ERROR: %sysfunc(sasmsg(smd_ds.bismsg,CSMART.APPLY_BIN_ERROR.2, noquote));
		  		%let msg2= "&cs_fact_lib.._&mm_ext_key._apply_bin_exp_act";
				%put &msg1 &msg2;  
               
				DATA &cs_fact_lib.._mm_ext_key._apply_bin_exp_act;
						set &debug_lib.._&model_key._&time_key._feed_table_act(where=(range_sk=.));
				RUN;

            %ABORT ABEND;
%end;
%bankfdn_rcset(&syserr);

*==================================================================;
* Deriving ACTUAL related information at bin level ;
*==================================================================;


proc sql;
  create table &debug_lib.._&model_key._&time_key._feed_table_act_1 as
    select
		a.range_sk,
      	count(a.record_rk) as no_of_records_actual,
	  	coalesce(sum(a.actual_good_bad),0) as no_of_actual_bads
	from &debug_lib.._&model_key._&time_key._feed_table_act as a
		 /* Subsetting the records for Applications that are ACCEPTED */
  	     %if &model_type=%str(&PD_APP_MODEL) %then %do;
           inner join dim.application_dim(where = ( kupcase(kcompress(outcome_cd)) = &OUTCOME_ACCEPT and valid_end_dttm = &valid_high_dttm.)) as b 
		   on (a.record_rk = b.application_rk)
		 %end;
	group by a.range_sk;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================================;
* Deriving DEV related information at bin level ;
*==================================================================;

/* START - Find VERSION_NO corresponding to current time_sk */


%if &isDevelopmentCycle = true %then %do;

	proc sql noprint;
		select max(range_proportion_version_no)
			into :range_proportion_version_no
		from &csb_apdm_libref..model_dev_measure_x_spec_map
		where model_rk = &mm_ext_key.;
	quit;

%end;
%else %do;

	%let time_sk_exist_flg = 0;
	proc sql noprint;
		select 1, dev_measure_version_no into :time_sk_exist_flg, :dev_version_no
		from csbfact.model_score_summary
		where model_rk = &model_key and score_time_sk = &time_key;
	quit;

	%if &time_sk_exist_flg = 1 %then %do;
		proc sql noprint;
			select range_proportion_version_no
					into :range_proportion_version_no
			from &csb_apdm_libref..model_dev_measure_x_spec_map
			where dev_measure_version_no = &dev_version_no
			and model_rk = &mm_ext_key.;
		quit;
	%end;
	%else %do;
		proc sql noprint;
			select max(range_proportion_version_no)
				into :range_proportion_version_no
			from &csb_apdm_libref..model_dev_measure_x_spec_map
			where model_rk = &mm_ext_key.;
		quit;
	%end;

%end;

/* END - Find VERSION_NO corresponding to current time_sk */

/* Find DEV related information */
proc sql;
  create table &debug_lib.._&model_key._&time_key._feed_table_dev as 
    select
		range_sk,
		no_of_records as no_of_records_dev,
		expected_pd as estimated_pd
	from &csb_apdm_libref..model_dev_feed_detail
	where model_rk = &mm_ext_key
	and version_no = &range_proportion_version_no;
quit;

%bankfdn_rcset(&sqlrc);


*==================================================================;
* Creating bin level feed ;
*==================================================================;

proc sql;

	CREATE TABLE &out_feed_lib..&feed_ds_nm_pool
	(
	MODEL_RK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL ,
	SCORE_TIME_SK NUMERIC(8) FORMAT = 12. INFORMAT = 12. NOT NULL ,
	POOL_SEQ_NO NUMERIC(8) NOT NULL,
	POOL_NAME CHARACTER(3000) FORMAT = $3000. INFORMAT = $3000.  ,
	NO_OF_RECORDS_DEV NUMERIC(8)  ,
	%if &model_type=%str(&PD_APP_MODEL) %then %do;
		NO_OF_RECORDS_ACTUAL_ALL NUMERIC(8)  ,
	%end;
	NO_OF_RECORDS_ACTUAL NUMERIC(8)  ,
	NO_OF_ACTUAL_BADS NUMERIC(8)  ,
	MAX_SCORE_POINTS NUMERIC(8) NOT NULL,
	ESTIMATED_PD  NUMERIC(8) ,
	LEVEL_SK NUMERIC(8) NOT NULL 
    );

    create unique index xpk_feed_pd on &out_feed_lib..&feed_ds_nm_pool(model_rk,score_time_sk,pool_seq_no);

quit;
%bankfdn_rcset(&sqlrc);

%if &model_type=%str(&PD_APP_MODEL) %then %do;

	proc sql;
	  create table &debug_lib.._&model_key._&time_key._feed_table_act_app as
	    select
			a.range_sk,
	      	count(a.record_rk) as no_of_records_actual_all
		  	from &debug_lib.._&model_key._&time_key._feed_table_act as a
			group by a.range_sk;
	quit;
%end;

proc sql;
  insert into &out_feed_lib..&feed_ds_nm_pool 
    select
		&model_key as model_rk,
	  	&time_key as score_time_sk,
		a.range_seq_no as pool_seq_no,
		a.range_name as pool_name,
      	coalesce(c.no_of_records_dev,0) as no_of_records_dev,
		%if &model_type=%str(&PD_APP_MODEL) %then %do;
			coalesce(d.no_of_records_actual_all,0) as no_of_records_actual_all,
		%end;
	  	coalesce(b.no_of_records_actual,0) as no_of_records_actual,
	  	coalesce(b.no_of_actual_bads,0) as no_of_actual_bads,
	  	a.max_score_points,
	  	c.estimated_pd,
	  	&m_level_sk as level_sk
	  	from &debug_lib.._&model_key._&time_key._rng_dtl_ext as a
				left join  
	     &debug_lib.._&model_key._&time_key._feed_table_act_1 as b
	on ( a.range_sk	= b.range_sk )
				left join
		&debug_lib.._&model_key._&time_key._feed_table_dev as c
	on ( a.range_sk = c.range_sk )
	%if &model_type=%str(&PD_APP_MODEL) %then %do;
				left join 
		&debug_lib.._&model_key._&time_key._feed_table_act_app d
			on ( a.range_sk = d.range_sk )
	%end;
	;
quit;
%bankfdn_rcset(&sqlrc);

%mend csbmva_feed_pd;
