/************************************************************************************************
       Module:  dabt_calc_rnt_var

     Function:  This macro applies the files obtained by dabt_create_files_for_rnt macro
				and aggregates the input dataset obtained by dabt_base_tbl_ext_for_rnt macro,
				D_<SOURCE>_<VAR_TYPE>_TIME_EXT, for a given source table

      Authors:  BIS Team
         Date:  06 October 2008
          SAS:  9.2

    Called-by:	Loop Job

        Calls:  1. dabt_create_files_for_rnt macro
				2. dabt_err_chk macro
				3. dabt_param_update macro

Datasets used:  LIB_STG.SOURCE_TABLE_INFO to obtain the subset query info.
				D_<SOURCE>_VAR_<VAR_TYPE> for obtaining the variable level info.
				D_<SOURCE>_<VAR_TYPE>_TIME_EXT is the input dataset
				that will be aggregated.

        Logic:   1. The input dataset, D_<SOURCE>_<VAR_TYPE>_TIME_EXT
					is filtered by applying the WHERE file created
					by dabt_create_files_for_rnt macro.
				 2. The variable level keys are obtained for the given variable level.
				 3. The aggregated dataset is obtained by applying the files created by
					dabt_create_files_for_rnt macro on the dataset obtained in step 1
					by grouping on the keys obtained in step 2. 
				 4. After each proc or data step the macro dabt_err_chk is called by passing
					SQL or DATA respectively, which checks for any error in the proc or
					data step and assigns the appropriate error code.
				 5. dabt_param_update macro is called to update the loop parameter list table
					with the highest return code (error code if any error occurred or 0, if
					there was no error).
				
   Parameters: INPUT:
				1. in_ds_nm			: Two level input dataset name. (respective variable list table)
				2. src		 		: Variable source table code, like 
										SNP for account_snapshot,
										TRN for account_transaction_fact and so on.
				3. trgt_tbl			: Target table sk where the variable is to be stored.
				4. var_type			: Type of variable, that is behavioural or semi behavioural.
				5. out_ds_nm		: Output dataset name for the aggregated dataset.
				6. subject_level_sk	: Subject level sk
				7. var_group_no		: group no identifying same selection criteria and time period (for RNT type of variables)

			OUTPUT:
				1. out_ds_nm	: Output dataset name for the aggregated dataset.
************************************************************************************************/

%macro dabt_calc_rnt_var(in_ds_nm=,src=,trgt_tbl=,var_type=,out_ds_nm=,subject_level_sk=, var_group_no=) / des='dabt_calc_rnt_var';

	/* Return code initialized to zero (this value changes if error occurs in any of the proc 
		or data step */
/*	%global job_rc;*/
	%let job_rc = 0;
	%let src = &src;
	%let var_type = &var_type;
	%let trgt_tbl = &trgt_tbl;
	%let var_group_no = &var_group_no;
	%let subject_level_sk = &subject_level_sk;
	%let in_ds_nm = &in_ds_nm;

	%if &m_deploy_abt_flg eq Y %then %do;

		%let input_var_ds_nm = &lib_scr_var..d_&src._var_&subject_level_sk._&var_group_no._&var_type.;
		/* Sort the variable list dataset and subset it for the given source, abt, level and order by date */
		proc sort data=&in_ds_nm(where=(variable_group_no = &var_group_no. and
										var_source_table_sk = &src. and abt_lvl_sk = &subject_level_sk. )) 
						out=&input_var_ds_nm.;
			by DERIVED_VAR_NM;
		run;

		%let rnt_usage_dt_sk = ;
		proc sql noprint;
			select put(rnt_usage_dt_sk,12.) into :rnt_usage_dt_sk
			from &lib_stg..job_list_dabt
			where job_id = "&job_id.";
		quit;
		/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
		%dabt_err_chk(type=SQL);

		%let rnt_usage_dt_sk = &rnt_usage_dt_sk;

		%let prv_level_no = %eval(&level_no - 1);
		%let in_ds_nm = ;
		proc sql noprint;
			select output_table_cd into :input_ds_nm
			from &lib_stg..job_list_dabt
			where level_no = &prv_level_no and run_no = &run_no. and
					kupcase(strip(var_type_cd)) = kupcase(strip("&var_type.")) and
					subject_level_sk = &subject_level_sk. and
					var_source_table_sk = &src. and rnt_usage_dt_sk = &rnt_usage_dt_sk.;
		quit;
		/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
		%dabt_err_chk(type=SQL);

		/* The variable level keys are obtained for the given variable level. */
		%let key = ;
		proc sql noprint;
			select level_key_column_nm into :key
			from &lib_stg..level_details 
			where level_sk = &subject_level_sk.;
		quit;
		/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
		%dabt_err_chk(type=SQL);

		data _null_;
			file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000;/* i18nOK:Line */ 
			length stmt_str $32767.;
			put '/* Drop output table before creating */';
			stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(out_ds_nm)_t1);";
			put stmt_str;
			put ;
			put ;
			stmt_str = '%let DABT_OUTPUT_TABLE_NM =' ||  "%superq(out_ds_nm)_t1;";
			put stmt_str;
			put ;
			stmt_str = '%let DABT_INPUT_TABLE_NM =' ||  strip("%superq(input_ds_nm)") || ";";
			put stmt_str;
			put ;
		run;
		/* Obtain the error code for the previous datastep executed,
				and retain the maximum error code */
		%dabt_err_chk(type=DATA);

		/* The following macro creates the files required for aggregation,
			such as the where, if, reset, retain files for the given source table,
			level and order by date*/
		%dabt_create_files_for_rnt(src=&src., trgt_tbl=&trgt_tbl., var_type=&var_type.,
						in_var_ds=&input_var_ds_nm., subject_level_sk=&subject_level_sk);			

		%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
			/* in case of rdbms if the source table has date fields then proc sort doesn't go complete inDB due to some format issues. 
				Hence taking max of the columns.*/
			%let keep_column = ;
			proc sql noprint;
				select catx(' ','MAX(',derived_var_nm,') as ',derived_var_nm) into :keep_column separated by ','
				from &lib_stg..variable_list_&var_type.
					where abt_lvl_sk = &subject_level_sk. and variable_group_no = &var_group_no;
			quit;
			/* Obtain the error code for the previous sql executed,
					and retain the maximum error code */
			%dabt_err_chk(type=SQL);


			%let keep_column = &keep_column;
		%end;
		%else %do;
			data _NULL_;
				call symput('key_com',prxchange('s/,/ /', -1, "&key"));/* i18nOK:Line */
			run;
			/* Obtain the error code for the previous sql executed,
				and retain the maximum error code */
			%dabt_err_chk(type=DATA);				
		%end;

		data _null_;
			file  "&m_deployed_code_path./&job_id..sas"  lrecl=64000 mod;/* i18nOK:Line */ 
			length stmt_str $32767.;
			put ;
			put '/* Check if there are multiple most recent records per subject*/';
			put '%let chk_dup_row = 0;';
			put 'proc sql noprint;';
			put '	select count(1) into :chk_dup_row';
			put "	from %superq(out_ds_nm)_t1";
			put "	group by &key";
			put "	having count(1) > 1;";
			put "quit;";
			put '%dabt_err_chk(type=SQL);';
			put '/* Drop output table before creating */';
			stmt_str = '%dabt_drop_table(m_table_nm=' || "%superq(out_ds_nm));";
			put stmt_str;
			put ;
			put '/* The below macro removes the duplicate rows per subject */';
			put ;
			put '%macro dabt_rnt_dup_rem;';
			put '	%if &chk_dup_row gt 0 %then %do;';
			put ;
			%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
				put '		/*If  multiple most recent records are present per subject then keep any one of them and drop others*/';
				put '		proc sql noprint;';
				put "			create table %superq(out_ds_nm) as ";
				put "			select &key, MAX(REFERENCE_DATE) as REFERENCE_DATE,";
				put "			MAX(REFERENCE_DTTM) as REFERENCE_DTTM, &keep_column";
				put "			from %superq(out_ds_nm)_t1";
				put "			group by &key;";
				put '		quit;';
				put '		%dabt_err_chk(type=SQL);';
			%end;
			%else %do;
				put '		/*If  multiple most recent records are present per subject then keep any one of them and drop others*/';
				put "		proc sort data=%superq(out_ds_nm)_t1 out=%superq(out_ds_nm) nodupkey;";
				put "			by &key_com;";
				put "		run;";
				put '		%dabt_err_chk(type=DATA);';
			%end;
			put ;
			put '			/* If the debug flag is set to N, then the temporary datasets and files are deleted */';
			stmt_str = '		%dabt_clean_if_debug_on(m_table_nm=' || "%superq(out_ds_nm)_t1);";
			put stmt_str;
			put ;
			put '	%end;';
			put '	%else %do;';
			put '		/*If multiple recent records are not present then simply rename existing table to required name*/';
			stmt_str = '		%let DABT_TABLE_NM = %sysfunc(scan(' || strip("%superq(out_ds_nm)") || ',2,.));';
			put stmt_str;
			put '		proc datasets lib=&lib_scr. nolist nodetails;';
	    	put '				change &DABT_TABLE_NM._t1 = &DABT_TABLE_NM;';
	    	put '		run;';
	    	put '		quit;';
			put '		%dabt_err_chk(type=DATA);';
			put '	%end;';
			put '%mend dabt_rnt_dup_rem;';
			put '%dabt_rnt_dup_rem;';
			put ;
			put ;
			put '/* Obtain the error code for the previous sql executed, and retain the maximum error code */';
			put '%dabt_err_chk(type=SQL);';
			put ;
			put '/* Obtain the error code for the previous datastep executed, and retain the maximum error code */';
			put '%dabt_err_chk(type=DATA);';
			put ;	
			put '/* Obtain the error code for the previous library statement executed, and retain the maximum error code */';
			put	'%dabt_err_chk(type=LIB);';
		run;
		/* Obtain the error code for the previous datastep executed,
				and retain the maximum error code */
		%dabt_err_chk(type=DATA);

		/* If the debug flag is set to N, then the temporary datasets created above are deleted */
		%if %kupcase("&DABT_DEBUG_FLG") eq "N" %then %do;
			proc sql noprint;
				drop table &input_var_ds_nm.;
				/* Obtain the error code for the previous sql executed,
						and retain the maximum error code */
				%dabt_err_chk(type=SQL);

			quit;
		%end;
	%end;
	%if &m_build_abt_flg eq Y %then %do;
		/* If the file exists then include the file that contains sql query to be executed.
		   This will create the table containing the subset population. 
		   If the file does not exists then set the job_rc flag to 5.*/
		%if %sysfunc(fileexist("&m_deployed_code_path./&job_id..sas")) %then %do;
			%include "&m_deployed_code_path./&job_id..sas" /lrecl=64000;
		%end;
		%else %do;
			%let job_rc = 5;
		%end;
	%end;
	

%mend dabt_calc_rnt_var;
