
/***************************************************************************************************************
       Module:  dabt_calculate_characteristics

     Function:  This macro computes the required characteristics from input dataset provided and writes to the
     			output dataset

      Authors:  BIS Team
         Date:  27 August 2013
          SAS:  9.4

    Called-by:	1. dabt_calc_chrstc_frm_lst_mdl_abt macro
    			2. dabt_calc_chrstc_frm_bld_abt macro
    			3. dabt_calc_chrstc_frm_dev_data macro

        Calls:  NA

        Logic:   1. Check whether all the required input parameters are passed.
        		 2. Get the characteristics for which computation is to be done.
				 3. Obtain the computation logic for the characteristics from apdm.
				 4. Create a file containing the query that computes the required characteristics.
				
   Parameters: 
				1. m_model_id				: Model Id
				2. m_input_ds_nm			: Two level input dataset name.
				3. m_bin_group_col_nm		: Bin group column name as it exists in the input dataset.
				4. m_bin_group_out_col_nm	: Bin group column name as required in the output dataset.
				5. m_calc_chrstc_for		: Parameter to define whether macro is called for calculation of 
												SCORECARD or BIN.
				6. m_bin_scheme_cd			: If macro is called for BIN, then this is mandatory parameter, which 
												should have the bin scheme code for which the characteristics
												computation is to be done.
				7. m_chrstc_cd_lst			: List of characteristics for which the computation is to be done.
												The list should be like for eg. 'TOT_PROP','PROP_BAD'
												If not passed then all the characterisitcs configured in the apdm
												for the given SCORECARD/BIN (for the given bin scheme) will get 
												computed. 
				8. m_temp_lib				: Libref, where the temp tables would be created.
				9. m_temp_file_path			: Path, where the temp file would be created.
			   10. m_output_ds_nm			: Two level output dataset name.
			   11. m_status_flg				: Macro which returns the status of the execution. This should contain
			   									name of the macro variable in which the status is to be returned.
			   									The values indicate:
			   									 0 for success, 1 for failure and 2 for insufficient inputs.

****************************************************************************************************************/

%macro dabt_calculate_characteristics(m_model_id=,m_input_ds_nm=,m_bin_group_col_nm=, 
											m_bin_group_out_col_nm =,
											m_calc_chrstc_for=SCORECARD, m_bin_scheme_cd=, m_chrstc_cd_lst =,
											m_temp_lib=,m_temp_file_path=, m_output_ds_nm=, m_status_flg=)/des='dabt_calculate_characteristics';


	%local job_rc sqlrc;
	
	%let m_model_id = &m_model_id;
	%let m_input_ds_nm = &m_input_ds_nm;
	%let m_bin_group_col_nm = &m_bin_group_col_nm;
	%let m_bin_group_out_col_nm = &m_bin_group_out_col_nm;
	%let m_calc_chrstc_for = %upcase(&m_calc_chrstc_for);
	%let m_bin_scheme_cd = &m_bin_scheme_cd;
	%let m_chrstc_cd_lst = %kupcase(%quote(&m_chrstc_cd_lst));
	%let m_chrstc_cd_lst = %sysfunc(tranwrd(%quote(&m_chrstc_cd_lst),%str(%"),%str(%')));
	%let m_output_ds_nm = &m_output_ds_nm;
	%let &m_status_flg = 0;
	
	%if "&m_model_id" eq "" or "&m_input_ds_nm" eq "" or "&m_bin_group_col_nm" eq "" %then %do;
		%let &m_status_flg = 2;
		/* return back as model id is not provided */
		%return;
	%end;
	%if "&m_calc_chrstc_for" eq "" %then %do;
		%let &m_status_flg = 2;
		/* return back as the type for which the characteristic is to be obtained (m_chrstc_for)
			that is BIN or SCRCD is not provided */
		%return;
	%end;
	%else %if "&m_calc_chrstc_for" eq "BIN" and "&m_bin_scheme_cd" eq "" %then %do;
		%let &m_status_flg = 2;
		/* return back as bin scheme is not provided */
		%return;
	%end;
	
	/* Obtain the model_sk for the given model_id */
	%let m_model_sk = .;
	proc sql noprint;
		select model_sk into :m_model_sk
		from &lib_apdm..model_master
		where model_id = "&m_model_id";
	quit;
	%dabt_err_chk(type=SQL);
	%let m_model_sk = &m_model_sk;
	
	%let m_bin_chrstc_sk_lst = ;
	/* Get the list of bin characteriestics that is to be computed */
	%if &m_calc_chrstc_for eq SCORECARD %then %do;
		/* For the given model, obtain the list of scorecard bin characteristics */
		proc sql noprint;
			select msc.bin_chrstc_sk into :m_bin_chrstc_sk_lst separated by ','
			from &lib_apdm..model_master mm 
				inner join &lib_apdm..model_x_scorecard_chrstc msc
				on (mm.model_sk = msc.model_sk and mm.model_sk = &m_model_sk.)
			%if "&m_chrstc_cd_lst" ne "" %then %do;
				inner join &lib_apdm..bin_characteristic bc
				on (msc.bin_chrstc_sk = bc.bin_chrstc_sk)
				where kupcase(bc.bin_chrstc_cd) in (&m_chrstc_cd_lst)
			%end;
				;
		quit;
		%dabt_err_chk(type=SQL);
		%let m_tmp_nm = bprp_&m_model_sk._scrcrd;
	%end;
	%else %if &m_calc_chrstc_for eq BIN %then %do;
		/* For the given model and bin scheme obtain the list of bin characteristics */
		%let m_bin_scheme_sk = ;
		proc sql noprint;
			/* Get the bin scheme sk from the given bin_scheme_cd for the given model */
			select mbas.bin_analysis_scheme_sk into :m_bin_scheme_sk
			from &lib_apdm..model_x_bin_analysis_scheme as mbas
				inner join &lib_apdm..bin_analysis_scheme_defn as bsb
				on (mbas.model_sk = &m_model_sk. and 
					mbas.bin_analysis_scheme_sk = bsb.bin_analysis_scheme_sk)
				where kupcase(strip(bin_analysis_scheme_cd)) = kupcase(strip("&m_bin_scheme_cd."));
			%dabt_err_chk(type=SQL);
				
			%let m_bin_scheme_sk = &m_bin_scheme_sk.;
			
			/* Get the list of characteristics */	
			select bsb.bin_chrstc_sk into :m_bin_chrstc_sk_lst separated by ','
			from &lib_apdm..bin_scheme_bin_chrstc_defn bsb
			%if "&m_chrstc_cd_lst" ne "" %then %do;
				inner join &lib_apdm..bin_characteristic bc
				on (bsb.bin_chrstc_sk = bc.bin_chrstc_sk)
				where kupcase(bin_chrstc_cd) in (&m_chrstc_cd_lst)
					 and bsb.bin_analysis_scheme_sk = &m_bin_scheme_sk.
			%end;
			%else %do;
				where bsb.bin_analysis_scheme_sk = &m_bin_scheme_sk.
			%end;
				;
			%dabt_err_chk(type=SQL);
		quit;
		%let m_tmp_nm = bprp_&m_model_sk._&m_bin_scheme_sk.;
	%end;
	
	%let m_bin_chrstc_sk_lst = &m_bin_chrstc_sk_lst;
	
	/* If characteristics exists for computaion then the following steps are executed */
	%if "&m_bin_chrstc_sk_lst" ne "" %then %do;
		%if %sysfunc(exist(&m_temp_lib..&m_tmp_nm.)) %then %do;
			proc sql noprint;
				drop table &m_temp_lib..&m_tmp_nm.;
			quit;
			%dabt_err_chk(type=SQL);
		%end;
		
		/* Get the separator for values. */
		%let m_separator=;
		data _null_;
			x= compress("&SEPARATOR_FOR_BIN_NOTATION.","'");
			call symput("m_separator",strip(x));
		run;
		%dabt_err_chk(type=DATA);
		
		/* Create a temp table containing the definition of the required characteristics based on the aggregation and filters defined. */
		proc sql noprint;
			create table &m_temp_lib..&m_tmp_nm. as
			select bc.bin_chrstc_sk, bc.bin_chrstc_cd, cctm.chrstc_calc_tmplt_cd,
					coalesce(vm.variable_column_nm, moc.scoring_output_column_nm) as chrstc_attrib_col_nm, 
					 upcase(atm.aggregation_type_cd) as chrstc_calc_agg_type_cd,
					 coalesce(vmf.variable_column_nm, mocf.scoring_output_column_nm) as chrstc_val_col_nm, 
					 tranwrd(bcfd.filter_value_txt, "&m_separator.", ",") as chrstc_calc_val, bcfd.filter_spec_type_cd,
					 bcfd.multi_condtn_boolean_oprtr_val, bcfd.sqnc_num as seq_no
			from &lib_apdm..bin_characteristic bc
				inner join &lib_apdm..characteristic_calc_tmplt_mstr cctm
				on (bc.chrstc_calc_template_sk = cctm.chrstc_calc_tmplt_sk)
				left join &lib_apdm..bin_chrstc_aggregation_dtl bcad
				on (bc.bin_chrstc_sk = bcad.bin_chrstc_sk)
				left join &lib_apdm..variable_master vm
				on (bcad.chrstc_msr_attrb_variable_sk = vm.variable_sk)
				left join &lib_apdm..model_output_column moc
				on (moc.model_sk = &m_model_sk. 
					and bcad.chrstc_msr_attrb_otpt_type_sk = moc.scoring_output_type_sk
					and bcad.chrstc_msr_attrb_otpt_type_sk is not null)
				left join &lib_apdm..aggregation_type_master atm
				on (bcad.aggregation_type_sk = atm.aggregation_type_sk)
				left join &lib_apdm..bin_chrstc_filter_dtl bcfd
				on (bc.bin_chrstc_sk = bcfd.bin_chrstc_sk)
				left join &lib_apdm..variable_master vmf
				on (bcfd.filter_operand_variable_sk = vmf.variable_sk)
				left join &lib_apdm..model_output_column mocf
				on (mocf.model_sk = &m_model_sk. 
					and bcfd.filter_operand_otpt_type_sk = mocf.scoring_output_type_sk
					and bcfd.filter_operand_otpt_type_sk is not null)
				where bc.bin_chrstc_sk in (&m_bin_chrstc_sk_lst)
				order by bc.bin_chrstc_sk, bcfd.filter_spec_type_cd, seq_no;
		quit;
		%dabt_err_chk(type=SQL);
					
		filename t_file "&m_temp_file_path./&m_tmp_nm..sas";
		
		/* Create query for computing total count of subjects in case of PROP_ACROSS_BIN */
		data _null_;
			file t_file  lrecl=64000;
			length stmt_str $32767.;
			set &m_temp_lib..&m_tmp_nm. end=last;
			by bin_chrstc_sk filter_spec_type_cd seq_no;
			where upcase(chrstc_calc_tmplt_cd) IN ('PROP_ACROSS_BIN', 'RATIO_ACROSS_BIN');
			retain tot_cnt filter_cnt 0 stmt_str;
			if _N_ = 1 then do;
				put 'proc sql noprint;';
				put '	select ';
			end;

			/* Assumption, that for PROP_ACROSS_BIN, the column filter_spec_type_cd will be null and there are no separate NUMERATOR or DENOMINATOR.*/
			if first.bin_chrstc_sk then do;
				tot_cnt = tot_cnt + 1;
				filter_cnt = 0;
			end;
			/* If filter exists then consider the filter while computing that variable */
			if strip(chrstc_calc_val) ne '' then do;
				filter_cnt = filter_cnt + 1;
				if filter_cnt = 1 then do;
					stmt_str = ' SUM(CASE WHEN ';
				end;
				/* multi_condtn_boolean_oprtr_val contains the 'AND'/'OR' value in case of multiple conditions, the first condition present has this as null value.*/
				stmt_str = strip(stmt_str) || " " || strip(multi_condtn_boolean_oprtr_val) || " " || strip(chrstc_val_col_nm) || ' IN (' || strip(chrstc_calc_val) || ')';  
			end;
			if last.bin_chrstc_sk then do;
				if filter_cnt = 0 then do;
					stmt_str = ' SUM(1)';
				end;
				else do;
					stmt_str = strip(stmt_str) || ' THEN 1 ELSE 0 END)';
				end;
			end;
			if last ne 1 then do;
				if last.bin_chrstc_sk then do;
					stmt_str = strip(stmt_str) || ',';
				end;
			end;
			if last.bin_chrstc_sk then do;
				put '		' stmt_str;
			end;
		run;
		%dabt_err_chk(type=DATA);
		
		/* Create query for assiging the total count of subjects in case of PROP_ACROSS_BIN to macro variables */
		data _null_;
			file t_file  lrecl=64000 mod;
			length stmt_str $32767.;
			set &m_temp_lib..&m_tmp_nm. end=last;
			by bin_chrstc_sk filter_spec_type_cd seq_no;
			where upcase(chrstc_calc_tmplt_cd) IN ('PROP_ACROSS_BIN', 'RATIO_ACROSS_BIN');
			retain tot_cnt 0 stmt_str;
			if _N_ = 1 then do;
				stmt_str =  ' into ';
			end;

			if first.bin_chrstc_sk then do;
				tot_cnt = tot_cnt + 1;
				stmt_str = strip(stmt_str) || " :m_tot_prop_cnt_" || strip(tot_cnt);  
			end;
			
			/* the statement is put only at the end.*/
			if last then do;
				put '	' stmt_str;
				put "	from &m_input_ds_nm;";
				put 'quit;';
				put ;
			end;
			else if last.bin_chrstc_sk then do;
				stmt_str = strip(stmt_str) || ',';
			end;
		run;
		%dabt_err_chk(type=DATA);
		
		/* Create query to obtain the characteristics */
		data _null_;
			file t_file  lrecl=64000 mod;
			length stmt_str stmt_str_num stmt_str_den $32767.;
			set &m_temp_lib..&m_tmp_nm. end=last;
			by bin_chrstc_sk filter_spec_type_cd seq_no;
			/* to use the macro variable created in the previous step for total proportion count. */
			retain tot_cnt filter_num_cnt filter_den_cnt 0 stmt_str_num stmt_str_den stmt_str;
			if _N_ = 1 then do;
				put ;
				put "proc sql noprint;";
				put "	create table &m_output_ds_nm as ";
				put "	select &m_bin_group_col_nm as &m_bin_group_out_col_nm,";
			end;
			/* Assumption: incase of PROP_ACROSS_BIN the value2 is always null, hence this will give
				 same result as for value1 */
			if chrstc_calc_tmplt_cd = 'PROP_ACROSS_BIN' or chrstc_calc_tmplt_cd = 'RATIO_ACROSS_BIN' then do;
				if first.bin_chrstc_sk then do;
					tot_cnt = tot_cnt + 1;
					filter_num_cnt = 0;
				end;
				if strip(chrstc_calc_val) ne '' then do;
					filter_num_cnt = filter_num_cnt + 1;
					if filter_num_cnt = 1 then do;
						stmt_str = ' SUM(CASE WHEN ';
					end;
					stmt_str = strip(stmt_str) || " " || strip(multi_condtn_boolean_oprtr_val) || " " || strip(chrstc_val_col_nm) || ' IN (' || strip(chrstc_calc_val) || ')';  
				end;
				if last.bin_chrstc_sk then do;
					if filter_num_cnt = 0 then do;
						stmt_str =  ' SUM(1) / %dabt_null_if_zero(&m_tot_prop_cnt_' || strip(tot_cnt)||')';
					end;
					else do;
						stmt_str = strip(stmt_str) || ' THEN 1 ELSE 0 END) / %dabt_null_if_zero(&m_tot_prop_cnt_' || strip(tot_cnt)||')';
					end;
					if chrstc_calc_tmplt_cd = 'PROP_ACROSS_BIN' then do;
						stmt_str = ' 100 * (' || strip(stmt_str) || ')';
					end;
					if last then do;
						stmt_str = strip(stmt_str) || ' as ' || strip(bin_chrstc_cd);
						put '		' stmt_str;
					end;
					else if last.bin_chrstc_sk then do;
						stmt_str = strip(stmt_str) || ' as ' || strip(bin_chrstc_cd) || ', ';
						put '		' stmt_str;
					end;
				end;
			end;
			else if chrstc_calc_tmplt_cd = 'PROP_WITHIN_BIN' or chrstc_calc_tmplt_cd = 'RATIO_WITHIN_BIN' then do;
				if first.bin_chrstc_sk then do;
					filter_num_cnt = 0;
					filter_den_cnt = 0;
				end;
				if strip(upcase(filter_spec_type_cd)) = 'NUMERATOR' then do;
					if strip(chrstc_calc_val) ne '' then do;
						filter_num_cnt = filter_num_cnt + 1;
						if filter_num_cnt = 1 then do;
							stmt_str_num = ' SUM(CASE WHEN ';
						end;
						stmt_str_num = strip(stmt_str_num) || " " || strip(multi_condtn_boolean_oprtr_val) || " " || strip(chrstc_val_col_nm) || ' IN (' || strip(chrstc_calc_val) || ')'; 
					end; 
				end;
				else if strip(upcase(filter_spec_type_cd)) = 'DENOMINATOR' then do;
					if strip(chrstc_calc_val) ne '' then do;	
						filter_den_cnt = filter_den_cnt + 1;
						if filter_den_cnt = 1 then do;
							stmt_str_den = ' %dabt_null_if_zero(SUM(CASE WHEN ';
						end;
						stmt_str_den = strip(stmt_str_den) || " " || strip(multi_condtn_boolean_oprtr_val) || " " || strip(chrstc_val_col_nm) || ' IN (' || strip(chrstc_calc_val) || ')';
					end;  
				end;
				
				if last.bin_chrstc_sk then do;
					if filter_num_cnt = 0 then do;
						stmt_str_num =  ' SUM(1)';
					end;
					else do;
						stmt_str_num = strip(stmt_str_num) || ' THEN 1 ELSE 0 END)';
					end;
					if filter_den_cnt = 0 then do;
						stmt_str_den =  ' SUM(1)';
					end;
					else do;
						stmt_str_den = strip(stmt_str_den) || ' THEN 1 ELSE 0 END))';
					end;
					stmt_str = strip(stmt_str_num) || ' / ' || strip(stmt_str_den);
					if chrstc_calc_tmplt_cd = 'PROP_WITHIN_BIN' then do;
						stmt_str = '100 * (' || strip(stmt_str) || ')';
					end;
					if last then do;
						stmt_str = strip(stmt_str) || ' as ' || strip(bin_chrstc_cd);
						put '		' stmt_str;
					end;
					else if last.bin_chrstc_sk then do;
						stmt_str = strip(stmt_str) || ' as ' || strip(bin_chrstc_cd) || ', ';
						put '		' stmt_str;
					end;
				end;
			end;
			else do;
				if first.bin_chrstc_sk then do;
					filter_num_cnt = 0;
				end;
				if strip(chrstc_calc_val) ne '' then do;
					filter_num_cnt = filter_num_cnt + 1;
					/* Average By Record */
					if chrstc_calc_agg_type_cd = 'AVC' then do;
						if filter_num_cnt = 1 then do;
							stmt_str = 'AVG( CASE WHEN ';
						end;
						/* Since there is no selection criteria, hence AVG will do average by record. */
						stmt_str = strip(stmt_str) || " " || strip(multi_condtn_boolean_oprtr_val) || " " || strip(chrstc_val_col_nm) || ' IN (' || strip(chrstc_calc_val) || ')';
					end;
					/* Count - counts the number of records in the given bin */
					else if chrstc_calc_agg_type_cd = 'CNT' then do;
						if filter_num_cnt = 1 then do;
							stmt_str = 'SUM( CASE WHEN ';
						end;
						stmt_str = strip(stmt_str) || " " || strip(multi_condtn_boolean_oprtr_val) || " " || strip(chrstc_val_col_nm) || ' IN (' || strip(chrstc_calc_val) || ')';
					end;
					/* Max / Min / Total */
					else do;
						if filter_num_cnt = 1 then do;
							stmt_str = strip(chrstc_calc_agg_type_cd) || '( ';
						end;
						stmt_str = strip(stmt_str) || " " || strip(multi_condtn_boolean_oprtr_val) || " " || strip(chrstc_val_col_nm) || ' IN (' || strip(chrstc_calc_val) || ')';  
					end;
				end;
				if last.bin_chrstc_sk then do;
					if filter_num_cnt = 0 then do;
							/* Average By Record */
						if chrstc_calc_agg_type_cd = 'AVC' then do;
							/* Since there is no selection criteria, hence AVG will do average by record. */
							stmt_str = 'AVG( ' || strip(chrstc_attrib_col_nm) || ')';
						end;
						/* Count - counts the number of records in the given bin */
						else if chrstc_calc_agg_type_cd = 'CNT' then do;
							stmt_str = 'SUM( 1 )';
						end;
						/* Max / Min / Total */
						else do;
							stmt_str = strip(chrstc_calc_agg_type_cd) || '( ' || strip(chrstc_attrib_col_nm) || ' )';
						end;
					end;
					else do;
						if strip(chrstc_attrib_col_nm) ne '' then do;
							stmt_str = strip(stmt_str) || ' THEN ' || strip(chrstc_attrib_col_nm) || ' ELSE . END)';
						end;
						else do;
							stmt_str = strip(stmt_str) || ' THEN 1 ELSE 0 END)';
						end;
					end;
					if last then do;
						stmt_str = strip(stmt_str) || ' as ' || strip(bin_chrstc_cd);
						put '		' stmt_str;
					end;
					else if last.bin_chrstc_sk then do;
						stmt_str = strip(stmt_str) || ' as ' || strip(bin_chrstc_cd) || ', ';
						put '		' stmt_str;
					end;
				end;
			end;
			if last then do;
				put "	from &m_input_ds_nm";
				put "	group by &m_bin_group_col_nm;";
				put 'quit;';
			end;
		run;
		%dabt_err_chk(type=DATA);
		
		%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
			option DBIDIRECTEXEC SQL_IP_TRACE=NOTE;
		%end;
	
		/* Execute query to obtain the characteristics */
		%include t_file /lrecl=64000;
		%dabt_err_chk(type=SQL);
		
		%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
			option NODBIDIRECTEXEC SQL_IP_TRACE=NONE;
		%end;
		
		%if %sysfunc(kupcase("&DABT_DEBUG_FLG")) eq "N" %then %do;
			proc sql noprint;
				drop table &m_temp_lib..&m_tmp_nm.;
			quit;
			
			/* here t_file is already assigned fileref */
			data _null_;
			    if fexist('t_file') then
			       rc=fdelete('t_file');
			run;
		%end;
	%end;
	%else %do;
		%let &m_status_flg = 1; 
	%end;
	
	%if &job_rc gt 4 %then %do;
		%let &m_status_flg = 1; 
	%end;
	
%mend dabt_calculate_characteristics;
